@@ -14,7 +14,7 @@ require 'forwardable'
 #   conn.get '/'
 #
 module Faraday
-  VERSION = "0.12.1"
+  VERSION = "0.13.0"
 
   class << self
     # Public: Gets or sets the root path that Faraday is being loaded from.
@@ -109,7 +109,7 @@ module Faraday
   #
   # Returns a Faraday::Connection, configured with the #default_adapter.
   def self.default_connection
-    @default_connection ||= Connection.new
+    @default_connection ||= Connection.new(default_connection_options)
   end
 
   # Gets the default connection options used when calling Faraday#new.
@@ -121,6 +121,7 @@ module Faraday
 
   # Public: Sets the default options used when calling Faraday#new.
   def self.default_connection_options=(options)
+    @default_connection = nil
     @default_connection_options = ConnectionOptions.from(options)
   end
 
